<#ftl strip_whitespace=true>

<#--<#macro renderEmptyElement text><!-- story element - ${text?html} was empty &ndash;&gt;</#macro>-->
<#macro renderMultiline text>${text?html?replace("\n", "<br/>")}</#macro>

<#macro renderMeta meta>
<div class="meta">
	<div class="keyword">${keywords.meta}</div>
    <#assign metaProperties=meta.getProperties()>
    <#list metaProperties.keySet() as name>
        <#assign property = metaProperties.get(name)>
		<div class="property">${keywords.metaProperty}${name?html} ${property?html}</div>
    </#list>
</div>
</#macro>

<#macro renderNarrative narrative>
<div class="narrative">
	<div class="keyword">${keywords.narrative}</div>
    <#assign isAlternative=narrative.isAlternative()?string>
    <#if isAlternative == "true">
		<div class="element asA"><span class="keyword asA">${keywords.asA}</span><span class="text"> ${narrative.asA}</span></div>
		<div class="element iWantTo"><span class="keyword iWantTo">${keywords.iWantTo}</span><span class="text"> ${narrative.iWantTo}</span>
		</div>
		<div class="element soThat"><span class="keyword soThat">${keywords.soThat}</span><span class="text"> ${narrative.soThat}</span>
		</div>
    <#else>
		<div class="element inOrderTo"><span class="keyword inOrderTo">${keywords.inOrderTo}</span><span
				class="text"> ${narrative.inOrderTo}</span></div>
		<div class="element asA"><span class="keyword asA">${keywords.asA}</span><span class="text"> ${narrative.asA}</span></div>
		<div class="element iWantTo"><span class="keyword iWantTo">${keywords.iWantTo}</span><span class="text"> ${narrative.iWantTo}</span>
		</div>
    </#if>
</div>
</#macro>

<#macro renderGivenStories givenStories>
<#--<div class="given-stories">-->
	<#--<div class="keyword">${keywords.givenStory} ${givenStory.path}</div>-->
    <#list givenStories as givenStory>
		<div class="given-story">
            <div class="title">
                <span class="keyword">${keywords.givenStory}</span> <span class="story-path">${givenStory.path}</span>
            </div>
            <div class="content">
                <#if givenStory.getScenarios()??>
                    <#list givenStory.getScenarios() as scenario>
                        <@renderScenario scenario/>
                    </#list>
                </#if>
            </div>
        </div>
    </#list>
<#--</div>-->
</#macro>

<#macro renderLifecycle lifecycle>
<div class="lifecycle">
	<div class="keyword">${keywords.lifecycle}</div>
    <#if !lifecycle.getBeforeSteps().isEmpty()>
		<div class="before">
			<div class="keyword">${keywords.before}</div>
            <#list lifecycle.getBeforeSteps() as step>
            <#--<div class="step">${step?html}</div>-->
                <@renderStep step/>
            </#list>
		</div>
    </#if>
    <#if !lifecycle.getAfterSteps().isEmpty()>
		<div class="after">
			<div class="keyword">${keywords.after}</div>
            <#list lifecycle.getOutcomes() as outcome>
				<div class="outcome">
                    <#if lifecycle.getAfterSteps(outcome)?has_content>
                    <#--TODO - there is no keyword currently for 'Outcome', will need to be localized below-->
						<div><span class="keyword">Outcome:</span><span class="text"> ${outcome}</span></div>
                        <#list lifecycle.getAfterSteps(outcome) as step>
                            <@renderStep step/>
                        </#list>
                    </#if>
				</div>
            </#list>
		</div>
    </#if>
</div>
</#macro>

<#macro renderScenario scenario>
    <#--<#if scenario.getGivenStories()??><@renderGivenStories scenario.getGivenStories()/></#if>-->
    <#if scenario.getOutcome() == "passed" || scenario.getOutcome() == "ignored" || scenario.getOutcome() == "not_performed">
        <#assign contentCollapsedOrExpanded = "collapsed">
    <#else>
        <#assign contentCollapsedOrExpanded = "expanded">
    </#if>
<div class="scenario ${scenario.getOutcome()}">
    <div class="title jb-story-ele-exp-trigger ${contentCollapsedOrExpanded}">
        <a href="#" onclick="jbStoryReportController.storyReportToggleElement(event, '.scenario');">
            <span class="keyword">${keywords.scenario}</span>
            <span class="text"> <@renderMultiline scenario.getTitle()/></span>
            <#if scenario.getOutcome() == "passed">
                <span class="aui-lozenge aui-lozenge-success" style="background-color:#14892c; border-color:#14892c; color:#fff">passed</span>
            <#elseif scenario.getOutcome() == "failed">
                <span class="aui-lozenge aui-lozenge-error" style="background-color:#d04437; border-color:#d04437; color:#fff">failed</span>
            <#elseif scenario.getOutcome() == "pending">
                <span class="aui-lozenge aui-lozenge-current" style="background-color:#f6c342; border-color:#f6c342; color:#fff">pending</span>
            <#elseif scenario.getOutcome() == "not_performed">
                <span class="aui-lozenge">not performed</span>
            <#elseif scenario.getOutcome() == "ignored">
                <span class="aui-lozenge aui-lozenge-moved">ignored</span>
            </#if>
        </a>
    </div>
    <div class="content ${contentCollapsedOrExpanded}">
        <#if scenario.getMeta()??><@renderMeta scenario.getMeta()/></#if>
        <#if scenario.getGivenStories()??><@renderGivenStories scenario.getGivenStories()/></#if>
        <#if scenario.getExamplesTable()??>
            <div class="examples">
                <div class="examples-keyword">${keywords.examplesTable}</div>
                <div class="examples-table">
                    <@renderTable scenario.getExamplesTable()/>
                </div>
            <#--examples themselves-->
                <#if scenario.getExamples()??>
                    <#list scenario.getExamples() as example>
                        <div class="example">
                            <div class="title">
                                <span class="keyword">${keywords.examplesTableRow}</span><span class="text"> ${example?html}</span>
                            </div>
                            <#assign steps = scenario.getStepsByExample(example)>
                            <#list steps as step>
                                <@renderStep step/>
                            </#list>
                        </div>
                    </#list>
                </#if>
            </div>
            <!-- end examples -->
        <#else> <!-- normal scenario steps -->
            <div class="steps">
                <#assign steps = scenario.getSteps()>
                    <#list steps as step>
                <@renderStep step/>
            </#list>
            </div>
        </#if>
    <#--<#if scenario.getPendingMethods()??>-->
    <#--<#list scenario.getPendingMethods() as method>-->
    <#--<div>-->
    <#--<pre class="pending">${method?html}</pre>-->
    <#--</div>-->
    <#--</#list>-->
    <#--</#if>-->
    </div>
</div>
<!-- end scenario -->
</#macro>

<#macro renderTable table>
    <#assign rows=table.getRows()>
    <#assign headers=table.getHeaders()>
    <#assign oddEven="odd">
<table>
	<thead>
	<tr>
        <#list headers as header>
			<th>${header?html}</th>
        </#list>
	</tr>
	</thead>
	<tbody>
        <#list rows as row>
		<tr class="${oddEven}">
            <#list headers as header>
                <#assign cell=row.get(header)>
				<td>${cell?html}</td>
            </#list>
		</tr>
            <#if oddEven == "odd">
                <#assign oddEven="even">
            <#else>
                <#assign oddEven="odd">
            </#if>
        </#list>
	</tbody>
</table>
</#macro>

<#macro renderOutcomeValue value dateFormat>
    <#if value?is_date>${value?string(dateFormat)}<#else>${value?string?replace("\n", "<br>")}</#if>
</#macro>

<#macro renderOutcomes table>
    <#assign outcomes=table.getOutcomes()>
    <#assign fields=table.getOutcomeFields()>
<span class="keyword">Outcomes:</span>
<table>
	<thead>
	<tr>
        <#list fields as field>
			<th>${field?html}</th>
        </#list>
	</tr>
	</thead>
	<tbody>
        <#list outcomes as outcome>
            <#assign isVerified=outcome.isVerified()?string>
            <#if isVerified == "true"> <#assign verified="verified"><#else><#assign verified="notVerified"></#if>
		<tr class="${verified}">
			<td style="width: 20%">${outcome.description?html}</td>
			<td clas="outcome-value"><@renderOutcomeValue outcome.getValue() table.getDateFormat()/></td>
			<td>${outcome.matcher?string}</td>
			<td><#if isVerified == "true">${keywords.yes}<#else>${keywords.no}</#if></td>
		</tr>
        </#list>
	</tbody>
</table>
</#macro>

<#macro renderStep step>
<#--<#assign formattedStep = step.getFormattedStep(EscapeMode.HTML, "<span class=\"step parameter\">{0}</span>")>-->
<div class="step ${step.outcome}">
<#--${formattedStep}-->

    <#list step.getStepTokens() as token>
        <#if token.getType() == "COMMENT">
            <span class="comment">${token.asString()?html}</span>
        <#elseif token.getType() == "KEYWORD">
			<span class="keyword">${token.asString()?html}</span>
        <#elseif token.getType() == "TEXT">
			<span class="text">${token.asString()?html}</span>
        <#elseif token.getType() == "PARAMETER">
			<span class="parameter">${token.asString()?html}</span>
        <#elseif token.getType() == "TABULAR_PARAMETER">
			<div class="tabular-parameter">
                <@renderTable token.getTable()/>
			</div>
        </#if>
    </#list>
    <@renderStepOutcome step/>
</div>
</#macro>

<#macro renderStepOutcome step>
<div class="step-outcome">
    <#assign outcome = step.getOutcome()>
    <#if outcome=="passed">
		<span class="outcome-keyword ${outcome}">(PASSED)</span>
    </#if>
    <#if outcome=="pending">
		<span class="outcome-keyword ${outcome}">(${keywords.pending})</span>
    </#if>
    <#if outcome=="failed">
		<span class="outcome-keyword ${outcome}">(${keywords.failed})</span>
    </#if>
    <#if outcome=="not_performed">
		<span class="outcome-keyword ${outcome}">(${keywords.notPerformed})</span>
    </#if>
    <#if outcome=="ignored">
		<span class="outcome-keyword ${outcome}">(${keywords.ignorable})</span>
    </#if>

    <#if step.getFailure()??>
    <#-- render failure trace-->
		<span class="step-failure">
            <#if step.getUnderlyingErrorMsg()??>
	            <span class="failure-msg">Underlying error - ${step.getUnderlyingErrorMsg()}</span>
            </#if>
			<a href="#" onclick="jbStoryReportController.toggleFailureTrace(event);"><span class="failure-show-trace-trigger"
			                                                       replace-text="hide trace">show trace</span></a>
			<div class="failure-trace" style="display: none;">
				<pre class="failure">${step.failureCause?html}</pre>
			</div>
        </span>
    </#if>

    <#if step.getOutcomes()??>
		<div class="outcomes">
            <@renderOutcomes step.getOutcomes()/>
		</div>
    </#if>

</div>
</#macro>

<#-- report start ======================================================================================================================-->

<div class="jb-story-report">

<#if storyExecutionTime??>
    <div class="storyExecutionTime">
        Story execution time: ${storyExecutionTime?html}
    </div>
</#if>

<#if story.getDescription()??>
	<div class="description">
        <@renderMultiline story.getDescription()/>
	</div>
</#if>

<#if story.getMeta()??><@renderMeta story.getMeta()/></#if>

<#if story.getNarrative()??><@renderNarrative story.getNarrative()/></#if>

<#if story.getGivenStories()??><@renderGivenStories story.getGivenStories()/></#if>

<#if story.getLifecycle()??><@renderLifecycle story.getLifecycle()/></#if>

<#if story.getScenarios()??>
    <#list story.getScenarios() as scenario>
        <@renderScenario scenario/>
    </#list>
</#if>

<#if story.isCancelled()?string == 'true'>
	<div class="cancelled">${keywords.storyCancelled} (${keywords.duration} ${story.storyDuration.durationInSecs} s)</div>
</#if>
</div>
<!-- end story -->

<#-- report end ========================================================================================================================-->

